const fs = require('fs');
const path = require('path');
const crypto = require('crypto');

class FilesystemMap {
    constructor(directory = 'fsMapData') {
        this.baseDir = directory;
        this.size = 0; // to track the size of all stored data
        if (!fs.existsSync(this.baseDir)) {
            fs.mkdirSync(this.baseDir);
        } else {
            process.log.debug(`Clearing CDN cache directory ${this.baseDir}`);
            this.clear();
        }
    }

    _getFilePath(key) {
        const hashedKey = crypto.createHash('sha256').update(String(key)).digest('hex');
        return path.join(this.baseDir, hashedKey);
    }

    _computeSize(value) {
        // Approximating the byte size of the string value
        return Buffer.from(value, 'utf8').length;
    }

    set(key, value) {
        const filePath = this._getFilePath(key);
        const dataSize = this._computeSize(value);
        if (this.has(key)) {
            // If key already exists, subtract its current size
            this.size -= this._computeSize(this.get(key));
        }
        fs.writeFileSync(filePath, value);
        this.size += dataSize;
    }

    get(key) {
        const filePath = this._getFilePath(key);
        if (fs.existsSync(filePath)) {
            return fs.readFileSync(filePath);
        }
        return undefined;
    }

    has(key) {
        return fs.existsSync(this._getFilePath(key));
    }

    delete(key) {
        const filePath = this._getFilePath(key);
        if (this.has(key)) {
            this.size -= this._computeSize(this.get(key));
            fs.unlinkSync(filePath);
        }
    }

    clear() {
        const files = fs.readdirSync(this.baseDir);
        for (const file of files) {
            fs.unlinkSync(path.join(this.baseDir, file));
        }
        this.size = 0;
    }

    roughSize() {
        return this.size;
    }
}

module.exports = FilesystemMap;